// Cube.cpp: implementation of the CCube class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Cube.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTexture* CCube::m_Texture = NULL;
GLuint CCube::m_Texture_num = -1;

CCube::CCube(double r, double x, double y, double z, double rx, double ry, double rz, double vx, double vy, double vz, double vrx, double vry, double vrz, bool changeable, bool transparent, unsigned char alpha, char * bmp_texture_file)
{
	m_r = r/2;
	m_rx = rx;
	m_ry = ry;
	m_rz = rz;
	m_x = x;
	m_y = y;
	m_z = z;
	m_vrx = vrx;
	m_vry = vry;
	m_vrz = vrz;
	m_vx = vx;
	m_vy = vy;
	m_vz = vz;
	m_changeable = changeable;
	m_transparent = transparent;

	if (bmp_texture_file && m_Texture_num == -1)
	{
		m_Texture = new CTexture();
		m_Texture->ReadFile(bmp_texture_file);
		m_Texture->BGRtoRGB();
		if(m_Texture->GetData())
		{
			glGenTextures(1, &m_Texture_num);
			glBindTexture(GL_TEXTURE_2D, m_Texture_num);
			if (m_transparent)
			{
				m_Texture->AddAlphaLayer(alpha);
				gluBuild2DMipmaps(GL_TEXTURE_2D, 4, m_Texture->GetWidth(), m_Texture->GetHeight(), GL_RGBA, GL_UNSIGNED_BYTE, m_Texture->GetData());
			}
			else
				gluBuild2DMipmaps(GL_TEXTURE_2D, 3, m_Texture->GetWidth(), m_Texture->GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, m_Texture->GetData());
		    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

			delete m_Texture;
			m_Texture = NULL;
		}
	}

	if (!m_changeable)
	{
		m_num_list = glGenLists(1);
		glNewList(m_num_list, GL_COMPILE_AND_EXECUTE);
			DrawCube();
		glEndList();
	}

}

CCube::~CCube()
{
}

void CCube::Draw(float x, float y, float z, bool multitexturing)
{
	glPushMatrix();	
		glTranslated(m_x - x, m_y - y, m_z - z);
		glRotated(m_rx, 1.0, 0.0, 0.0);
		glRotated(m_ry, 0.0, 1.0, 0.0);
		glRotated(m_rz, 0.0, 0.0, 1.0);
		if (multitexturing)
		{
			GLfloat amplitude = 1.0f;

			glColor3f(amplitude, amplitude, amplitude);

			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			glEnable(GL_BLEND);
			glDepthFunc(GL_LEQUAL);
			glDepthMask(GL_TRUE);
			glDisable(GL_LIGHTING);
			glDisable(GL_DITHER);
			glShadeModel(GL_FLAT);
//			glShadeModel(GL_SMOOTH);
			glEnable(GL_TEXTURE_2D);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);

			glEnable(GL_LIGHTING);
			glDisable(GL_TEXTURE_2D);
			glDepthFunc(GL_EQUAL);
			glDepthMask(GL_FALSE);
			glEnable(GL_DITHER);
			glBlendFunc(GL_ONE, GL_ONE);
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);

		}
		else
			if (m_changeable)
				DrawCube();
			else
				glCallList(m_num_list);
	glPopMatrix();
}

void CCube::Update(int t)
{
	m_x += m_vx * t;
	m_y += m_vy * t;
	m_z += m_vz * t;

	m_rx += m_vrx * t;
	m_ry += m_vry * t;
	m_rz += m_vrz * t;

}

void CCube::DrawCube()
{
	if (m_Texture_num != -1)
		glBindTexture(GL_TEXTURE_2D, m_Texture_num);

    glBegin(GL_QUADS);
		glNormal3d(0.0,0.0,1.0);
		glTexCoord2f(0.0, 1.0); glVertex3d(-m_r,  m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d(-m_r, -m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, -m_r, m_r);
		glTexCoord2f(0.0, 0.0); glVertex3d( m_r,  m_r, m_r);

		glNormal3d(0.0,0.0,-1.0);
		glTexCoord2f(0.0, 1.0); glVertex3d(  m_r,  m_r, -m_r);
		glTexCoord2f(0.0, 0.0); glVertex3d(  m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( -m_r,  m_r, -m_r);

		glNormal3d(1.0,0.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, -m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( m_r, m_r, -m_r);

		glNormal3d(-1.0,0.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( -m_r, -m_r, m_r);

		glNormal3d(0.0,-1.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, -m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( -m_r, -m_r, -m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, -m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( m_r, -m_r, m_r);

		glNormal3d(0.0,1.0,0.0);
		glTexCoord2f(0.0, 0.0); glVertex3d( -m_r, m_r, m_r);
		glTexCoord2f(1.0, 0.0); glVertex3d( m_r, m_r, m_r);
		glTexCoord2f(1.0, 1.0); glVertex3d( m_r, m_r, -m_r);
		glTexCoord2f(0.0, 1.0); glVertex3d( -m_r, m_r, -m_r);
	glEnd();
/*	GLUquadricObj *obj = gluNewQuadric();
	gluSphere(obj, 7, 20, 20);
	gluDeleteQuadric(obj);*/
}

